/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2013-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXSimulation
 * @author     Nikolaus Vahrenkamp
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _SIMULATOR_SelfLocalizationDynamicSimulation_H_
#define _SIMULATOR_SelfLocalizationDynamicSimulation_H_

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/services/tasks/PeriodicTask.h>
#include <RobotAPI/interface/core/RobotState.h>

// MemoryX
#include <MemoryX/core/entity/Entity.h>
#include <MemoryX/libraries/memorytypes/entity/AgentInstance.h>
#include <MemoryX/interface/component/WorkingMemoryInterface.h>

// Simulator
#include <ArmarXSimulation/interface/simulator/SimulatorInterface.h>

#include <VirtualRobot/VirtualRobot.h>


namespace armarx
{
    class SelfLocalizationDynamicSimulationPropertyDefinitions:
        public ComponentPropertyDefinitions
    {
    public:
        SelfLocalizationDynamicSimulationPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            defineOptionalProperty<std::string>("SimulatorName", "Simulator", "Name of the simulator component that should be used");
            defineRequiredProperty<std::string>("RobotName", "Name of the robot in the simulator (as specified in the robot's xml file)");
            defineOptionalProperty<std::string>("AgentName", "", "Name of the agent instance. If empty, RobotName property is used");
            defineOptionalProperty<std::string>("RobotStateComponentName", "RobotStateComponent", "Name of the RobotStateComponent that should be used");
            defineOptionalProperty<std::string>("WorkingMemoryName", "WorkingMemory", "Name of the WorkingMemory that should be used");
            defineOptionalProperty<bool>("ContinuousMode", true, "If true, the robots pose is being written to the memory continuously. Otherwise step has to be called. ");
        }
    };


    /**
     * SelfLocalizationDynamicSimulation uses the armarx simulator to retrieve thre current psoe of the robot and reports it to the agent layer.
     */
    class SelfLocalizationDynamicSimulation:
        virtual public ::armarx::Component,
        virtual public SelfLocalizationDynamicSimulationInterface
    {
    public:

        SelfLocalizationDynamicSimulation();

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual armarx::PropertyDefinitionsPtr createPropertyDefinitions()
        {
            return armarx::PropertyDefinitionsPtr(new SelfLocalizationDynamicSimulationPropertyDefinitions(getConfigIdentifier()));
        }

        virtual void onInitComponent();
        virtual void onConnectComponent();
        virtual void onDisconnectComponent();
        virtual void onExitComponent() {}

        /**
        * @see Component::getDefaultName()
        *
        */
        std::string getDefaultName() const
        {
            return "SelfLocalizationDynamicSimulation";
        }

        virtual void step(const Ice::Current& c = ::Ice::Current()) override;


    protected:

        PeriodicTask<SelfLocalizationDynamicSimulation>::pointer_type execTask;

    private:

        void periodicExec();

        void reportRobotPose();

        int cycleTime;

        SimulatorInterfacePrx simulatorProxy;
        RobotStateComponentInterfacePrx robotStateComponentPrx;

        memoryx::WorkingMemoryInterfacePrx memoryPrx;
        memoryx::AgentInstancesSegmentBasePrx agentsMemoryPrx;
        memoryx::AgentInstancePtr robotAgent;

        std::string robotAgentId;
        std::string robotName;

        Mutex dataMutex;
    };
}

#endif
